@@ -229,6 +229,7 @@ class VacBot():
         self.vacuum_status = None
         self.fan_speed = None
         self.water_level = None
+        self.mop_attached: bool = False
 
         # Populated by component Lifespan reports
         self.components = {}
@@ -261,6 +262,15 @@ class VacBot():
         self.thread_livemap = threading.Thread(target=self.refresh_liveMap, daemon=False, name="schedule_thread_livemap")
         self.thread_components = threading.Thread(target=self.refresh_components, daemon=False, name="schedule_thread_components")
 
+        self.errorEvents = EventEmitter()
+        self.lifespanEvents = EventEmitter()
+        self.fanspeedEvents = EventEmitter()
+        self.cleanLogsEvents = EventEmitter()
+        self.waterEvents = EventEmitter()          
+        self.batteryEvents = EventEmitter()
+        self.statusEvents = EventEmitter()
+        self.statsEvents = EventEmitter()
+
     def connect_and_wait_until_ready(self):
         self.iotmq.connect_and_wait_until_ready()
         self.iotmq.schedule(30, self.send_ping)
@@ -279,6 +289,8 @@ class VacBot():
         if not error == '':
             _LOGGER.warning("*** error = " + error)
 
+        self.errorEvents.notify(event)            
+
     def _handle_life_span(self, event):
         response = event['body']['data'][0]
         type = response['type']
@@ -295,6 +307,8 @@ class VacBot():
        
         self.components[type] = lifespan
 
+        self.lifespanEvents.notify(event)
+
     def _handle_fan_speed(self, event):
         response = event['body']['data']
         speed = response['speed']
@@ -306,17 +320,24 @@ class VacBot():
       
         self.fan_speed = speed
 
+        self.fanspeedEvents.notify(self.fan_speed)
+
     def _handle_clean_logs(self, event):
         response = event.get('logs')
         self.lastCleanLogs = []
 
-        # Ecovacs API is changing their API, this request may not working properly
-        if response is not None and len(response) >= 0:
-            self.last_clean_image = response[0]['imageUrl']
-            for cleanLog in response:
-                self.lastCleanLogs.append({'timestamp': cleanLog['ts'], 'imageUrl': cleanLog['imageUrl'],
+        try:
+            # Ecovacs API is changing their API, this request may not working properly
+            if response is not None and len(response) >= 0:
+                self.last_clean_image = response[0]['imageUrl']
+                for cleanLog in response:
+                    self.lastCleanLogs.append({'timestamp': cleanLog['ts'], 'imageUrl': cleanLog['imageUrl'],
                                            'type': cleanLog['type']})
-                                           
+
+            self.cleanLogsEvents.notify(event=(self.lastCleanLogs, self.last_clean_image))
+        except:
+            _LOGGER.warning("No last clean image found")
+
     def _handle_water_info(self, event):
         response = event['body']['data']
         amount = response['amount']
@@ -327,11 +348,14 @@ class VacBot():
             _LOGGER.warning("Unknown water level: '" + str(amount) + "'")
       
         self.water_level = amount
+        self.mop_attached = bool(response.get("enable"))
+
+        self.waterEvents.notify(event=(self.water_level, self.mop_attached))
 
     def _handle_clean_report(self, event):
         response = event['body']['data']
         if response['state'] == 'clean':
-            if response['trigger'] == 'app':
+            if response['trigger'] == 'app' or response['trigger'] == 'shed':
                 if response['cleanState']['motionState'] == 'working':
                     self.vacuum_status = 'STATE_CLEANING'
                 elif response['cleanState']['motionState'] == 'pause':
@@ -340,6 +364,8 @@ class VacBot():
                     self.vacuum_status = 'STATE_RETURNING'
             elif response['trigger'] == 'alert':
                 self.vacuum_status = 'STATE_ERROR'
+        
+        self.statusEvents.notify(self.vacuum_status)
 
     def _handle_map_trace(self, event):
         response = event['body']['data']
@@ -424,6 +450,8 @@ class VacBot():
         except ValueError:
             _LOGGER.warning("couldn't parse battery status " + response)
 
+        self.batteryEvents.notify(self.battery_status)
+
     def _handle_charge_state(self, event):
         response = event['body']
         status = 'none'
@@ -443,6 +471,8 @@ class VacBot():
         if status != 'none':
             self.vacuum_status = status
 
+        self.statusEvents.notify(self.vacuum_status)
+
     def _handle_stats(self, event):
         response = event['body']
 
@@ -461,6 +491,8 @@ class VacBot():
         else:
             _LOGGER.error("Error in finding stats, status code = " + response['code']) #Log this so we can identify more errors    
 
+        self.statsEvents.notify(event)
+
     def _vacuum_address(self):
         return self.vacuum['did']
 
@@ -528,16 +560,16 @@ class VacBot():
             _LOGGER.warning("*** Error condition: " + err.condition)
 
     def request_all_statuses(self):
-        if not self.thread_statuses.isAlive():
+        if not self.thread_statuses.is_alive():
             self.thread_statuses = threading.Thread(target=self.refresh_statuses, daemon=False, name="schedule_thread_statuses")
             self.thread_statuses.start()
 
         if self.live_map_enabled:
-            if not self.thread_livemap.isAlive():
+            if not self.thread_livemap.is_alive():
                 self.thread_livemap = threading.Thread(target=self.refresh_liveMap, daemon=False, name="schedule_thread_livemap")
                 self.thread_livemap.start()
                 
-        if not self.thread_components.isAlive():
+        if not self.thread_components.is_alive():
             self.thread_components = threading.Thread(target=self.refresh_components, daemon=False, name="schedule_thread_components")
             self.thread_components.start()
     
@@ -634,4 +666,31 @@ class VacBotCommand:
         return self.command_name() + " command"
 
     def command_name(self):
-        return self.__class__.__name__.lower()
\ No newline at end of file
+        return self.__class__.__name__.lower()
+
+class EventEmitter(object):
+    """A very simple event emitting system."""
+    def __init__(self):
+        self._subscribers = []
+
+    def subscribe(self, callback):
+        listener = EventListener(self, callback)
+        self._subscribers.append(listener)
+        return listener
+
+    def unsubscribe(self, listener):
+        self._subscribers.remove(listener)
+
+    def notify(self, event):
+        for subscriber in self._subscribers:
+            subscriber.callback(event)
+
+
+class EventListener(object):
+    """Object that allows event consumers to easily unsubscribe from events."""
+    def __init__(self, emitter, callback):
+        self._emitter = emitter
+        self.callback = callback
+
+    def unsubscribe(self):
+        self._emitter.unsubscribe(self)
